@@ -60,7 +60,7 @@ struct pcre_context {
         prelude_timer_t timer;
         pcre_context_setting_t *setting;
 
-        idmef_message_t *idmef;
+        pcre_state_t *state;
 };
 
 
@@ -220,7 +220,7 @@ static int parse_rule_optregex(pcre_plugin_t *plugin, pcre_rule_t *rule, const c
 
 
 
-static pcre_rule_container_t *search_rule(prelude_list_t *head, int id)
+static pcre_rule_container_t *search_rule(prelude_list_t *head, unsigned int id)
 {
         prelude_list_t *tmp;
         pcre_rule_container_t *cur;
@@ -241,7 +241,7 @@ static pcre_rule_container_t *search_rule(prelude_list_t *head, int id)
 
 
 
-static int add_goto_single(pcre_plugin_t *plugin, pcre_rule_t *rule, int id, prelude_bool_t optional)
+static int add_goto_single(pcre_plugin_t *plugin, pcre_rule_t *rule, unsigned int id, prelude_bool_t optional)
 {
         pcre_rule_container_t *new, *cur;
 
@@ -269,9 +269,10 @@ static int add_goto_single(pcre_plugin_t *plugin, pcre_rule_t *rule, int id, pre
 
 static int add_goto(pcre_plugin_t *plugin, pcre_rule_t *rule, const char *idstr, prelude_bool_t optional)
 {
-        int ret, i, idmin = 0, idmax = 0;
+        int ret;
+        unsigned int i, idmin = 0, idmax = 0;
 
-        ret = sscanf(idstr, "%d-%d", &idmin, &idmax);
+        ret = sscanf(idstr, "%u-%u", &idmin, &idmax);
         if ( ret < 1 ) {
                 prelude_log(PRELUDE_LOG_WARN, "could not parse goto value '%s'.\n", idstr);
                 return -1;
@@ -571,7 +572,7 @@ static int parse_rule_keyword(pcre_plugin_t *plugin, pcre_rule_t *rule,
                               const char *filename, int line,
                               const char *keyword, const char *value)
 {
-        int i;
+        unsigned int i;
         struct {
                 const char *keyword;
                 int (*func)(pcre_plugin_t *plugin, pcre_rule_t *rule, const char *value);
@@ -831,7 +832,7 @@ static int parse_ruleset(prelude_list_t *head, pcre_plugin_t *plugin, const char
 
 
 
-static void pcre_run(prelude_plugin_instance_t *pi, const lml_log_source_t *ls, const lml_log_entry_t *log_entry)
+static void pcre_run(prelude_plugin_instance_t *pi, const lml_log_source_t *ls, lml_log_entry_t *log_entry)
 {
         int ret;
         prelude_list_t *tmp;
@@ -982,8 +983,8 @@ static void _pcre_context_destroy(pcre_context_t *ctx)
         if ( ctx->setting )
                 pcre_context_setting_destroy(ctx->setting);
 
-        if ( ctx->idmef )
-                idmef_message_destroy(ctx->idmef);
+        if ( ctx->state )
+                pcre_state_destroy(ctx->state);
 
         prelude_timer_destroy(&ctx->timer);
         prelude_list_del(&ctx->list);
@@ -994,11 +995,23 @@ static void _pcre_context_destroy(pcre_context_t *ctx)
 
 
 
+static void alert_from_context(pcre_state_t *state)
+{
+        pcre_state_push_idmef(state, state->idmef);
+
+        /*
+         * Set various information, detect_time/analyzer.
+         */
+        lml_alert_set_infos(state->idmef, state->le);
+        lml_alert_emit(NULL, NULL, state->idmef);
+}
+
+
 void pcre_context_destroy(pcre_context_t *ctx)
 {
-        if ( ctx->setting->flags & PCRE_CONTEXT_SETTING_FLAGS_ALERT_ON_DESTROY && ctx->idmef ) {
+        if ( ctx->setting->flags & PCRE_CONTEXT_SETTING_FLAGS_ALERT_ON_DESTROY && ctx->state->idmef ) {
                 prelude_log_debug(1, "[%s]: emit alert on destroy.\n", ctx->name);
-                lml_alert_emit(NULL, NULL, ctx->idmef);
+                alert_from_context(ctx->state);
         }
 
         _pcre_context_destroy(ctx);
@@ -1010,9 +1023,9 @@ static void pcre_context_expire(void *data)
 {
         pcre_context_t *ctx = data;
 
-        if ( ctx->setting->flags & PCRE_CONTEXT_SETTING_FLAGS_ALERT_ON_EXPIRE && ctx->idmef ) {
+        if ( ctx->setting->flags & PCRE_CONTEXT_SETTING_FLAGS_ALERT_ON_EXPIRE && ctx->state->idmef ) {
                 prelude_log_debug(1, "[%s]: emit alert on expire.\n", ctx->name);
-                lml_alert_emit(NULL, NULL, ctx->idmef);
+                alert_from_context(ctx->state);
         }
 
         _pcre_context_destroy(ctx);
@@ -1022,13 +1035,19 @@ static void pcre_context_expire(void *data)
 
 idmef_message_t *pcre_context_get_idmef(pcre_context_t *ctx)
 {
-        return ctx->idmef;
+        return ctx->state->idmef;
 }
 
 
-
-int pcre_context_new(pcre_plugin_t *plugin, const char *name, idmef_message_t *idmef, pcre_context_setting_t *setting)
+pcre_state_t *pcre_context_get_state(pcre_context_t *ctx)
 {
+        return ctx->state;
+}
+
+
+int pcre_context_new(pcre_plugin_t *plugin, const char *name, pcre_state_t *state, pcre_context_setting_t *setting)
+{
+        int ret;
         pcre_context_t *ctx;
 
         if ( ! (setting->flags & PCRE_CONTEXT_SETTING_FLAGS_QUEUE) ) {
@@ -1059,6 +1078,13 @@ int pcre_context_new(pcre_plugin_t *plugin, const char *name, idmef_message_t *i
                 return -1;
         }
 
+        ret = pcre_state_clone(state, &ctx->state);
+        if ( ret < 0 ) {
+                free(ctx->name);
+                free(ctx);
+                return -1;
+        }
+
         setting->refcount += 1;
         ctx->setting = setting;
         prelude_timer_init_list(&ctx->timer);
@@ -1070,9 +1096,6 @@ int pcre_context_new(pcre_plugin_t *plugin, const char *name, idmef_message_t *i
                 prelude_timer_init(&ctx->timer);
         }
 
-        if ( idmef )
-                ctx->idmef = idmef_message_ref(idmef);
-
         prelude_list_add_tail(&plugin->context_list, &ctx->list);
 
         return 0;
