@@ -27,7 +27,9 @@
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <inttypes.h>
 #include <sys/types.h>
+#include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -35,6 +37,10 @@
 
 #include <gcrypt.h>
 
+#ifdef HAVE_SYS_FILIO_H
+# include <sys/filio.h>
+#endif
+
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
@@ -86,15 +92,15 @@
 /*
  * Logfile metadata stuff.
  */
-#define METADATA_SIZE 12
 #define METADATA_CHECKSUM_SIZE 4
+#define METADATA_SIZE (sizeof(off_t) + METADATA_CHECKSUM_SIZE)
 #define METADATA_CHECKSUM_TYPE GCRY_MD_CRC32
 
 #define LOG_LINE_MAXSIZE 65535
 
 #define LOGFILE_RENAME_CLASS   "Log file rename"
 #define LOGFILE_DELETION_CLASS "Log file deletion"
-#define LOGFILE_DELETION_IMPACT "An attacker might have erased the logfile,"               \
+#define LOGFILE_DELETION_IMPACT "An attacker might have erased the logfile, "               \
                                 "or a log rotation program may have rotated the logfile."
 
 #define LOGFILE_MODIFICATION_CLASS "Log file inconsistency"
@@ -111,22 +117,27 @@ typedef struct {
         FILE *metadata_fd;
         prelude_bool_t need_position;
 
-        uint64_t last_rotation_size;
+        off_t last_rotation_size;
         time_t last_rotation_time;
         time_t last_rotation_time_interval;
 
         prelude_string_t *buf;
 
-        uint64_t need_more_read;
-        uint64_t last_size;
+        off_t need_more_read;
+        off_t last_size;
+        size_t current_line_len;
+
+        union {
+                ev_stat st;
+                ev_io io;
+        } event;
 
-        ev_stat event;
         int prev_errno;
 } monitor_fd_t;
 
 
+static void libev_io_cb(ev_io *io, int revents);
 static void libev_stat_cb(ev_stat *st, int revents);
-static int check_logfile_data(monitor_fd_t *monitor, ev_statdata *st);
 void _lml_handle_signal_if_needed(void);
 
 extern lml_config_t config;
@@ -134,8 +145,8 @@ extern lml_config_t config;
 
 static PRELUDE_LIST(active_fd_list);
 static PRELUDE_LIST(inactive_fd_list);
-static unsigned int max_rotation_size_offset = DEFAULT_MAX_ROTATION_SIZE_OFFSET;
 static unsigned int max_rotation_time_offset = DEFAULT_MAX_ROTATION_TIME_OFFSET;
+static unsigned int max_rotation_size_offset = DEFAULT_MAX_ROTATION_SIZE_OFFSET;
 static file_server_metadata_flags_t metadata_flags = FILE_SERVER_METADATA_FLAGS_LAST;
 
 
@@ -222,10 +233,10 @@ static void logfile_alert(monitor_fd_t *fd, ev_statdata *st_old, ev_statdata *st
         if ( ret < 0 )
                 goto err;
 
-        if ( stat_to_file(fd, st_old, target, IDMEF_FILE_CATEGORY_ORIGINAL) < 0 )
+        if ( st_old && stat_to_file(fd, st_old, target, IDMEF_FILE_CATEGORY_ORIGINAL) < 0 )
                 goto err;
 
-        if ( stat_to_file(fd, st_new, target, IDMEF_FILE_CATEGORY_CURRENT) < 0 )
+        if ( st_new && stat_to_file(fd, st_new, target, IDMEF_FILE_CATEGORY_CURRENT) < 0 )
                 goto err;
 
         ret = idmef_alert_new_assessment(alert, &assessment);
@@ -244,7 +255,7 @@ static void logfile_alert(monitor_fd_t *fd, ev_statdata *st_old, ev_statdata *st
 
 
 
-static void logfile_modified_alert(monitor_fd_t *monitor, ev_statdata *st_old, ev_statdata *st_new)
+static void logfile_modified_alert(monitor_fd_t *monitor, ev_statdata *st_old, struct stat *st_new)
 {
         int ret;
         prelude_string_t *str;
@@ -286,7 +297,53 @@ static void logfile_modified_alert(monitor_fd_t *monitor, ev_statdata *st_old, e
 }
 
 
-static int file_metadata_read(monitor_fd_t *monitor, uint64_t *start, unsigned char **buf)
+/*
+ * This function return -2 if it couldn't read a full syslog line,
+ * or -1 if an error occured.
+ *
+ * The number of character that has been read is returned otherwise.
+ */
+static off_t read_logfile(monitor_fd_t *fd, off_t *available)
+{
+        int ret;
+        off_t i = 0;
+
+        while ( 1 ) {
+                ret = getc(fd->fd);
+                if ( ret == EOF ) {
+                        clearerr(fd->fd);
+                        *available -= i;
+                        return -1;
+                }
+
+                i++;
+                fd->current_line_len++;
+
+                if ( ret == '\n' )
+                        break;
+
+                if ( fd->current_line_len <= LOG_LINE_MAXSIZE ) {
+                        ret = prelude_string_ncat(fd->buf, (const char *) &ret, 1);
+                        if ( ret < 0 )
+                                prelude_log(PRELUDE_LOG_ERR, "error buffering input: %s.\n", prelude_strerror(ret));
+                }
+
+                else if ( fd->current_line_len == LOG_LINE_MAXSIZE + 1 )
+                        prelude_log(PRELUDE_LOG_WARN, "line too long (internal limit of %u characters).\n", LOG_LINE_MAXSIZE);
+
+                if ( i == *available ) {
+                        *available = 0;
+                        return -2;
+                }
+        }
+
+        *available -= i;
+        return i;
+}
+
+
+
+static int file_metadata_read(monitor_fd_t *monitor, off_t *start, unsigned char **buf)
 {
         ssize_t ret;
         struct stat st;
@@ -308,14 +365,14 @@ static int file_metadata_read(monitor_fd_t *monitor, uint64_t *start, unsigned c
         if ( ret != METADATA_SIZE )
                 return -1;
 
-        *start = prelude_align_uint64(*buf);
-        *buf = *buf + sizeof(uint64_t);
+        memcpy(start, *buf, sizeof(*start));
+        *buf = *buf + sizeof(off_t);
 
         return 0;
 }
 
 
-static int file_metadata_save(monitor_fd_t *monitor, uint64_t offset)
+static int file_metadata_save(monitor_fd_t *monitor, off_t offset)
 {
         int ret;
         unsigned char buf[METADATA_SIZE];
@@ -343,23 +400,22 @@ static int file_metadata_save(monitor_fd_t *monitor, uint64_t offset)
 
 
 
-static int verify_metadata_checksum(const char *log, unsigned char *metadata_sum)
+static int verify_metadata_checksum(prelude_string_t *log, unsigned char *metadata_sum)
 {
         unsigned char sum[METADATA_CHECKSUM_SIZE];
 
         assert(gcry_md_get_algo_dlen(METADATA_CHECKSUM_TYPE) == sizeof(sum));
-        gcry_md_hash_buffer(METADATA_CHECKSUM_TYPE, sum, log, strlen(log) - 1);
+        gcry_md_hash_buffer(METADATA_CHECKSUM_TYPE, sum, prelude_string_get_string(log), prelude_string_get_len(log));
 
         return memcmp(sum, metadata_sum, sizeof(sum));
 }
 
 static int file_metadata_get_position(monitor_fd_t *monitor)
 {
-        uint64_t offset = 0;
-        char buf[8192];
         struct stat st;
         const char *filename;
         int ret, have_metadata = 0;
+        off_t offset = 0, available = LOG_LINE_MAXSIZE;
         unsigned char msum[METADATA_SIZE], *sumptr = msum;
 
         filename = lml_log_source_get_name(monitor->source);
@@ -378,7 +434,7 @@ static int file_metadata_get_position(monitor_fd_t *monitor)
 
         if ( ! have_metadata ) {
                 prelude_log(PRELUDE_LOG_INFO, "%s: No metadata available, starting from tail.\n", filename);
-                return fseek(monitor->fd, st.st_size, SEEK_SET);
+                return fseeko(monitor->fd, st.st_size, SEEK_SET);
         }
 
         if ( st.st_size < offset ) {
@@ -387,7 +443,7 @@ static int file_metadata_get_position(monitor_fd_t *monitor)
                 return 0;
         }
 
-        ret = fseek(monitor->fd, offset, SEEK_SET);
+        ret = fseeko(monitor->fd, offset, SEEK_SET);
         if ( ret < 0 ) {
                 prelude_log(PRELUDE_LOG_ERR, "%s: error seeking to offset %" PRELUDE_PRIu64 ": %s.\n",
                             filename, offset, strerror(errno));
@@ -397,14 +453,17 @@ static int file_metadata_get_position(monitor_fd_t *monitor)
         /*
          * If the metadata checksum does not match, the file was probably rotated.
          */
-        if ( ! fgets(buf, sizeof(buf), monitor->fd) || verify_metadata_checksum(buf, sumptr) != 0 ) {
+        if ( read_logfile(monitor, &available) < 0 || verify_metadata_checksum(monitor->buf, sumptr) != 0 ) {
                 prelude_log(PRELUDE_LOG_INFO, "%s: log file was rotated, starting from head.\n", filename);
                 monitor->last_size = 0;
-                return fseek(monitor->fd, 0, SEEK_SET);
+                monitor->current_line_len = 0;
+                prelude_string_clear(monitor->buf);
+                return fseeko(monitor->fd, 0, SEEK_SET);
         }
 
-        monitor->last_size = offset;
-        monitor->last_size += strlen(buf);
+        prelude_string_clear(monitor->buf);
+        monitor->last_size = offset + monitor->current_line_len;
+        monitor->current_line_len = 0;
 
         prelude_log(PRELUDE_LOG_INFO, "%s: resuming log analyzis at offset %" PRELUDE_PRIu64 ".\n",
                     filename, monitor->last_size);
@@ -448,103 +507,78 @@ static int file_metadata_open(monitor_fd_t *monitor)
 }
 
 
-
-
-/*
- * This function return -1 if it couldn't read a full syslog line,
- * or if an error occured.
- *
- * The size of the whole syslog line is returned otherwise (not only what
- * has been read uppon this call).
- */
-static off_t read_logfile(monitor_fd_t *fd, off_t available)
-{
-        char c;
-        int ret;
-        size_t i = 0;
-        prelude_bool_t ignore_remaining = FALSE;
-
-        while ( 1 ) {
-                if ( i == LOG_LINE_MAXSIZE ) {
-                        prelude_log(PRELUDE_LOG_WARN, "line too long (internal limit of %u characters).\n",
-                                    LOG_LINE_MAXSIZE);
-                        ignore_remaining = TRUE;
-                }
-
-                ret = getc(fd->fd);
-                if ( ret == EOF ) {
-                        clearerr(fd->fd);
-                        return -1;
-                }
-
-                i++;
-
-                if ( ret == '\n' )
-                        break;
-
-                if ( ! ignore_remaining ) {
-                        c = (char) ret;
-
-                        ret = prelude_string_ncat(fd->buf, &c, 1);
-                        if ( ret < 0 ) {
-                                prelude_log(PRELUDE_LOG_ERR, "error buffering input: %s.\n", prelude_strerror(ret));
-                                return -1;
-                        }
-                }
-
-                if ( i == available )
-                        return -2;
-        }
-
-        return i;
-}
-
-
-
-static int check_logfile_data(monitor_fd_t *monitor, ev_statdata *st)
+static int check_logfile_data(monitor_fd_t *monitor, struct stat *st)
 {
         off_t len, ret;
         int eventno = 0;
+        size_t slen = 0;
 
-        if ( monitor->fd != stdin && ! monitor->need_more_read && st->st_size == monitor->last_size )
+        if ( ! monitor->need_more_read && st->st_size == monitor->last_size )
                 return 0;
 
         len = (st->st_size - monitor->last_size) + monitor->need_more_read;
         monitor->last_size = st->st_size;
 
-        while ( (ret = read_logfile(monitor, len)) >= 0 ) {
+        while ( len && (ret = read_logfile(monitor, &len)) >= 0 ) {
 
                 eventno++;
-                config.line_processed++;
 
                 /*
                  * If the line we read only contained a '\n', string and len will be 0.
                  */
-                if ( prelude_string_get_len(monitor->buf) ) {
-                        lml_dispatch_log(monitor->source,
-                                         prelude_string_get_string(monitor->buf),
-                                         prelude_string_get_len(monitor->buf));
-
-                        file_metadata_save(monitor, st->st_size - len);
+                if ( (slen = prelude_string_get_len(monitor->buf)) ) {
+                        lml_dispatch_log(monitor->source, prelude_string_get_string(monitor->buf), slen);
+                        file_metadata_save(monitor, ftello(monitor->fd) - monitor->current_line_len);
                 }
 
-                len -= prelude_string_get_len(monitor->buf) + 1; /* +1 account for the '\n' */
+                monitor->current_line_len = 0;
 
                 prelude_string_clear(monitor->buf);
                 _lml_handle_signal_if_needed();
-
-                if ( len == 0 )
-                        break;
         }
 
-        if ( ret == -2 )
-                len = 0; /* everything has been read, but line is not complete */
-
         /*
-         * if len isn't 0, it mean we got EOF before reading every new byte,
-         * we want to retry reading even if st_size isn't modified then.
+         * if len is not 0, we got EOF before reading every new byte, we
+         * will try reading the data on next invocation, even if st_size
+         * isn't modified.
          */
         monitor->need_more_read = len;
+        return eventno;
+}
+
+
+static int check_stdin_data(monitor_fd_t *monitor)
+{
+        size_t slen;
+        off_t ret, len;
+        int eventno = 0, bytes;
+
+        if ( config.batch_mode )
+                len = LOG_LINE_MAXSIZE;
+        else {
+                ret = ioctl(STDIN_FILENO, FIONREAD, &bytes);
+                if ( ret < 0 ) {
+                        prelude_log(PRELUDE_LOG_ERR, "FIONREAD failed on stdin: %s.\n", strerror(errno));
+                        return -1;
+                }
+
+                len = (off_t) bytes;
+        }
+
+        while ( len && (ret = read_logfile(monitor, &len)) >= 0 ) {
+
+                eventno++;
+
+                /*
+                 * If the line we read only contained a '\n', string and len will be 0.
+                 */
+                if ( (slen = prelude_string_get_len(monitor->buf)) ) {
+                        lml_dispatch_log(monitor->source, prelude_string_get_string(monitor->buf), slen);
+                }
+
+                prelude_string_clear(monitor->buf);
+                _lml_handle_signal_if_needed();
+        }
 
         return eventno;
 }
@@ -571,10 +605,16 @@ static monitor_fd_t *monitor_new(lml_log_source_t *ls)
                 return NULL;
         }
 
-        ev_stat_init(&new->event, libev_stat_cb, lml_log_source_get_name(new->source), 1);
-        new->event.data = new;
+        if ( strcmp(lml_log_source_get_name(ls), STDIN_FILENAME) != 0 ) {
+                ev_stat_init(&new->event.st, libev_stat_cb, lml_log_source_get_name(new->source), 1);
+                new->event.st.data = new;
+                ev_stat_start(&new->event.st);
+        } else {
+                ev_io_init(&new->event.io, libev_io_cb, STDIN_FILENO, EV_READ);
+                new->event.io.data = new;
+                ev_io_start(&new->event.io);
+        }
 
-        ev_stat_start(&new->event);
         prelude_list_add(&inactive_fd_list, &new->list);
 
         return new;
@@ -582,7 +622,7 @@ static monitor_fd_t *monitor_new(lml_log_source_t *ls)
 
 
 
-static int monitor_set_position(monitor_fd_t *monitor, const char *filename, int fd)
+static int monitor_set_position(monitor_fd_t *monitor, const char *filename)
 {
         int ret = 0;
 
@@ -596,12 +636,12 @@ static int monitor_set_position(monitor_fd_t *monitor, const char *filename, int
 
         else if ( metadata_flags & FILE_SERVER_METADATA_FLAGS_HEAD ) {
                 prelude_log(PRELUDE_LOG_INFO, "%s: user requested starting from head.\n", filename);
-                ret = fseek(monitor->fd, 0, SEEK_SET);
+                ret = fseeko(monitor->fd, 0, SEEK_SET);
         }
 
         else if ( metadata_flags & FILE_SERVER_METADATA_FLAGS_TAIL ) {
                 prelude_log(PRELUDE_LOG_INFO, "%s: user requested starting from tail.\n", filename);
-                ret = fseek(monitor->fd, 0, SEEK_END);
+                ret = fseeko(monitor->fd, 0, SEEK_END);
         }
 
         if ( ret < 0 ) {
@@ -618,9 +658,13 @@ static int monitor_set_position(monitor_fd_t *monitor, const char *filename, int
 static int monitor_open(monitor_fd_t *monitor)
 {
         int ret, fd;
-        const char *filename;
+        const char *filename = lml_log_source_get_name(monitor->source);
 
-        filename = lml_log_source_get_name(monitor->source);
+        ret = prelude_string_new(&monitor->buf);
+        if ( ret < 0 ) {
+                prelude_log(PRELUDE_LOG_ERR, "could not create string object: %s.\n", prelude_strerror(ret));
+                return ret;
+        }
 
         if ( strcmp(filename, STDIN_FILENAME) == 0 )
                 monitor->fd = stdin;
@@ -637,6 +681,7 @@ static int monitor_open(monitor_fd_t *monitor)
 #endif
 
                         monitor->prev_errno = errno;
+                        prelude_string_destroy(monitor->buf);
                         return -1;
                 }
 
@@ -646,20 +691,16 @@ static int monitor_open(monitor_fd_t *monitor)
                 fcntl(fd, F_SETFD, fcntl(fd, F_GETFD) | FD_CLOEXEC);
 #endif
 
-                ret = monitor_set_position(monitor, filename, fd);
-                if ( ret < 0 )
+                ret = monitor_set_position(monitor, filename);
+                if ( ret < 0 ) {
+                        prelude_string_destroy(monitor->buf);
                         return -1;
+                }
 
                 if ( ret == 0 )
                         prelude_log(PRELUDE_LOG_INFO, "%s: now available for monitoring.\n", filename);
         }
 
-        ret = prelude_string_new(&monitor->buf);
-        if ( ret < 0 ) {
-                prelude_log(PRELUDE_LOG_ERR, "could not create string object: %s.\n", prelude_strerror(ret));
-                return ret;
-        }
-
         monitor->need_more_read = 0;
         prelude_list_del(&monitor->list);
         prelude_list_add_tail(&active_fd_list, &monitor->list);
@@ -676,6 +717,7 @@ static void monitor_close(monitor_fd_t *monitor)
         fclose(monitor->fd);
         monitor->fd = NULL;
         monitor->last_size = 0;
+        monitor->current_line_len = 0;
 
         prelude_list_del(&monitor->list);
         prelude_list_add_tail(&inactive_fd_list, &monitor->list);
@@ -689,13 +731,17 @@ static void monitor_close(monitor_fd_t *monitor)
  * cause heavy performance problem. The best solution may be to centralize
  * the logging on a remote host.
  */
-static void check_modification_time(monitor_fd_t *monitor, ev_statdata *prev, ev_statdata *st)
+static void check_modification_time(monitor_fd_t *monitor, ev_statdata *prev, struct stat *st)
 {
         int ret;
+        const char *filename;
 
         if ( st->st_mtime >= prev->st_mtime && st->st_size >= monitor->last_size )
                 return; /* everythings sound okay */
 
+        filename = lml_log_source_get_name(monitor->source);
+
+        prelude_log(PRELUDE_LOG_INFO, "%s: has been %s.\n", filename, (st->st_size >= monitor->last_size) ? "modidifed" : "truncated");
         logfile_modified_alert(monitor, prev, st);
 
         /*
@@ -703,88 +749,91 @@ static void check_modification_time(monitor_fd_t *monitor, ev_statdata *prev, ev
          * descriptor to EOF, and start analyzing from this place.
          */
         if ( st->st_size < monitor->last_size ) {
-                ret = fseek(monitor->fd, 0, SEEK_END);
+                ret = fseeko(monitor->fd, 0, SEEK_END);
                 if ( ret < 0 ) {
-                        prelude_log(PRELUDE_LOG_ERR, "%s: error seeking to end of file: %s.\n",
-                                    lml_log_source_get_name(monitor->source), strerror(errno));
+                        prelude_log(PRELUDE_LOG_ERR, "%s: error seeking to end of file: %s.\n", filename, strerror(errno));
                         return;
                 }
 
                 monitor->need_more_read = 0;
                 monitor->last_size = st->st_size;
+                monitor->current_line_len = 0;
         }
 }
 
 
-static int get_rotation_size_offset(monitor_fd_t *monitor, ev_statdata *st)
+static off_t get_rotation_size_offset(monitor_fd_t *monitor, ev_statdata *st)
 {
-        off_t diff;
-        int prev = 0;
-
-        diff = MAX(monitor->last_rotation_size, st->st_size) -
-               MIN(monitor->last_rotation_size, st->st_size);
+        off_t diff = 0;
 
         if ( monitor->last_rotation_size )
-                prev = 1;
+                diff = (off_t) imaxabs((intmax_t) monitor->last_rotation_size - st->st_size);
 
         monitor->last_rotation_size = st->st_size;
 
-        return prev ? diff : 0;
+        return diff;
 }
 
 
-static int get_rotation_time_offset(monitor_fd_t *monitor, ev_statdata *st)
+static time_t get_rotation_time_offset(monitor_fd_t *monitor)
 {
-        int prev = 0;
-        time_t interval, now, offset;
+        time_t interval = 0, diff = 0, now = time(NULL);
 
-        now = time(NULL);
-        interval = now - monitor->last_rotation_time;
+        if ( monitor->last_rotation_time )
+                interval = now - monitor->last_rotation_time;
 
-        offset = MAX(monitor->last_rotation_time_interval, interval) -
-                 MIN(monitor->last_rotation_time_interval, interval);
-
-        if ( monitor->last_rotation_time ) {
-                prev = 1;
-                monitor->last_rotation_time_interval = interval;
-        }
+        if ( interval && monitor->last_rotation_time_interval )
+                diff = (time_t) imaxabs((intmax_t) monitor->last_rotation_time_interval - interval);
 
         monitor->last_rotation_time = now;
+        monitor->last_rotation_time_interval = interval;
 
-        return prev ? offset : 0;
+        return diff;
 }
 
 
-
-static int is_file_already_used(monitor_fd_t *monitor, ev_statdata *st_old, ev_statdata *st_new)
+static int is_file_already_used(monitor_fd_t *monitor, ev_statdata *st_prev, ev_statdata *st_cur, struct stat *st_now)
 {
         int ret;
         char buf[1024];
-        int toff, soff;
-        struct stat st;
+        off_t soff;
+        time_t toff;
         const char *filename, *ctxt;
         idmef_impact_t *impact;
         idmef_classification_t *classification;
         prelude_string_t *str;
+        prelude_bool_t is_deleted;
 
         filename = lml_log_source_get_name(monitor->source);
 
-        if ( st_new->st_nlink > 0 )
+        /*
+         * rename = 1 - 0
+         * rename + re-create = 1 - 1 - !=
+         * delete = 0 - 0
+         * delete + re-create = 0 - 1
+         */
+        if ( st_now->st_nlink > 0 && st_cur->st_nlink > 0 && st_now->st_ino == st_cur->st_ino )
                 return 0;
 
-        fstat(fileno(monitor->fd), &st);
-
         /*
          * test if the file has been removed
          */
-        if ( st.st_nlink > 0 && st_new->st_nlink == 0 ) {
+        if ( st_now->st_nlink > 0 ) {
                 prelude_log(PRELUDE_LOG_INFO, "%s: has been renamed.\n", filename);
                 ctxt = LOGFILE_RENAME_CLASS;
+                is_deleted = FALSE;
         } else {
                 prelude_log(PRELUDE_LOG_INFO, "%s: has been deleted.\n", filename);
                 ctxt = LOGFILE_DELETION_CLASS;
+                is_deleted = TRUE;
         }
 
+        /*
+         * Before closing the monitor, handle any unread data.
+         */
+        if ( st_now->st_size > monitor->last_size )
+                check_logfile_data(monitor, st_now);
+
         monitor_close(monitor);
 
         ret = idmef_classification_new(&classification);
@@ -808,55 +857,92 @@ static int is_file_already_used(monitor_fd_t *monitor, ev_statdata *st_old, ev_s
         idmef_impact_set_type(impact, IDMEF_IMPACT_TYPE_FILE);
         idmef_impact_set_completion(impact, IDMEF_IMPACT_COMPLETION_SUCCEEDED);
 
-        soff = get_rotation_size_offset(monitor, st_new);
-        toff = get_rotation_time_offset(monitor, st_new);
+        soff = get_rotation_size_offset(monitor, st_now);
+        toff = get_rotation_time_offset(monitor);
 
-        if ( toff <= max_rotation_time_offset|| soff <= max_rotation_size_offset ) {
+        if ( toff <= max_rotation_time_offset || soff <= max_rotation_size_offset ) {
                 idmef_impact_set_severity(impact, IDMEF_IMPACT_SEVERITY_INFO);
                 ret = idmef_impact_new_description(impact, &str);
+                if ( ret < 0 )
+                        return ret;
                 prelude_string_set_constant(str, LOGFILE_DELETION_IMPACT);
         } else {
                 idmef_impact_set_severity(impact, IDMEF_IMPACT_SEVERITY_HIGH);
                 ret = idmef_impact_new_description(impact, &str);
+                if ( ret < 0 )
+                        return ret;
                 snprintf(buf, sizeof(buf), "An inconsistency has been observed in file rotation: "
-                        "The differences between the previously observed rotation time and size are higher "
-                        "than the allowed limits: size difference=%u bytes allowed=%u bytes, time "
-                        "difference=%u seconds allowed=%u seconds", soff, max_rotation_size_offset,
-                         toff, max_rotation_time_offset);
+                        "the differences between the previously observed rotation size and time are higher "
+                        "than allowed limits: %" PRELUDE_PRIu64 " bytes difference (%" PRELUDE_PRIu64" allowed), "
+                        "%" PRELUDE_PRIu64 " seconds difference (%" PRELUDE_PRIu64" allowed)",
+                         (intmax_t) soff, (intmax_t) max_rotation_size_offset,
+                         (intmax_t) toff, (intmax_t) max_rotation_time_offset);
                 prelude_string_set_ref(str, buf);
         }
 
-        logfile_alert(monitor, st_old, st_new, classification, impact);
+        if ( is_deleted )
+                logfile_alert(monitor, st_now, NULL, classification, impact);
+        else
+                logfile_alert(monitor, st_prev, st_now, classification, impact);
 
         return -1;
 }
 
 
+static void libev_io_cb(ev_io *io, int revents)
+{
+        int ret;
+
+        ret = check_stdin_data(io->data);
+        if ( ret <= 0 )
+                ev_io_stop(io);
+}
+
 
 static void libev_stat_cb(ev_stat *st, int revents)
 {
         int ret;
+        struct stat fst;
         monitor_fd_t *monitor = st->data;
 
         if ( ! monitor->fd )
                 if ( monitor_open(monitor) < 0 )
                         return;
 
-        if ( monitor->fd != stdin ) {
-                ret = is_file_already_used(monitor, &st->prev, &st->attr);
-                if ( ret < 0 )
+        /*
+         * Do not rely on libev statistics gathered by stat().
+         *
+         * If a file is written then deleted, but re-created very fast,
+         * only fstat() can report an st_nlink of 0.
+         *
+         * If a file is deleted from libev point of view, only fstat()
+         * will be able to provide latest information about the stat of
+         * the file.
+         */
+        fstat(fileno(monitor->fd), &fst);
+
+        ret = is_file_already_used(monitor, &st->prev, &st->attr, &fst);
+        if ( ret < 0 ) {
+                if ( st->attr.st_nlink == 0 )
                         return;
+                else {
+                        /*
+                         * The file has been deleted then created again: trigger opening/reading of the new dfile.
+                         */
+                        return libev_stat_cb(st, revents);
+                }
         }
 
         /*
-         * check mtime consistency.
+         * check mtime/size consistency.
          */
-        check_modification_time(monitor, &st->prev, &st->attr);
+        check_modification_time(monitor, &st->prev, &fst);
+
 
         /*
          * read and analyze available data.
          */
-        check_logfile_data(monitor, &st->attr);
+        check_logfile_data(monitor, &fst);
 }
 
 
@@ -881,7 +967,11 @@ int file_server_read_once(void)
         prelude_list_for_each_safe(&active_fd_list, tmp, bkp) {
                 monitor = prelude_list_entry(tmp, monitor_fd_t, list);
 
-                event = check_logfile_data(monitor, &monitor->event.attr);
+                if ( monitor->fd != stdin )
+                        event = check_logfile_data(monitor, &monitor->event.st.attr);
+                else
+                        event = check_stdin_data(monitor);
+
                 if ( event > 0 )
                         ret = event;
         }
@@ -891,27 +981,27 @@ int file_server_read_once(void)
 
 
 
-void file_server_set_max_rotation_time_offset(unsigned int val)
+void file_server_set_max_rotation_time_offset(time_t val)
 {
         max_rotation_time_offset = val;
 }
 
 
 
-unsigned int file_server_get_max_rotation_time_offset(void)
+time_t file_server_get_max_rotation_time_offset(void)
 {
         return max_rotation_time_offset;
 }
 
 
 
-void file_server_set_max_rotation_size_offset(unsigned int val)
+void file_server_set_max_rotation_size_offset(off_t val)
 {
         max_rotation_size_offset = val;
 }
 
 
-unsigned int file_server_get_max_rotation_size_offset(void)
+off_t file_server_get_max_rotation_size_offset(void)
 {
         return max_rotation_size_offset;
 }
