@@ -26,6 +26,6 @@
 
 #include "regex.h"
 
-void lml_dispatch_log(regex_list_t *list, lml_log_source_t *ls, const char *str, size_t size);
+void lml_dispatch_log(lml_log_source_t *ls, const char *str, size_t size);
 
 #endif				/* COMMON_H */
